//
//  main.c
//  carbon_ibrc
//
//  Created by Zefir Kurtisi on 13.04.2005.
//

#include <Carbon/Carbon.h>

#include "ibrc_client.h"


/* Konstanten Interface-Builder
 */
#define	SIGNATURE	'IBRC'
#define	ID_LINE		128
#define	ID_BUTTON	129
#define	ID_LOG		130
#define ID_NAME		131
#define	ID_NICK		132
#define	ID_CHANNEL	133


static WindowRef static_win; // Referenz wird statisch gesichert


/*
 * Ausgabe einer Zeile in das Chat-Fenster
 * Anmerkung: Textflaeche nicht scrollbar -> neue Zeile wird oben
 *            ausgegeben.
 ****************************************************************
 */
void print_line(CFMutableStringRef msg_text)
{
	const ControlID k_pad={SIGNATURE, ID_LOG};
	ControlRef pad;

	CFStringRef old_pad_text;
	
	// Zeit in CString umwandeln
	Str255 timestr;
	long time_date;
	GetDateTime(&time_date);
	TimeString(time_date, 1, timestr, NULL);
	
	// alten Text holen
	GetControlByID(static_win, &k_pad, &pad);
	GetControlData(pad,0,kControlEditTextCFStringTag,sizeof(CFStringRef),&old_pad_text,NULL);
 
	// neuer_text = uhrzeit|  neue_textzeile
	//              alter_text
	CFMutableStringRef pad_text=CFStringCreateMutable(NULL, 0);
	CFStringAppendCString(pad_text,timestr, 0);
	CFStringAppendCString(pad_text,"|  ", 0);
	CFStringAppend(pad_text,msg_text);
	CFStringAppend(pad_text,old_pad_text);

	CFStringRef new_string=CFStringCreateCopy(NULL,pad_text);
	SetControlData(pad,0,kControlEditTextCFStringTag,sizeof(CFStringRef),&new_string);

	// Ressourcen freigeben
	CFRelease(pad_text);
	CFRelease(new_string);
	
	// Text neu darstellen
	DrawOneControl(pad);
}


/*
 * Hilfsfunktion zur zeilenweisen Ausgabe von Nachrichten
 * WICHTIG: neue Zeile wird mit \r codiert !!!
 ********************************************************
 */
void outMessage(char *from, char *sep, char *msg)
{
	const ControlID k_pad={SIGNATURE, ID_LOG};

	CFMutableStringRef msg_text=CFStringCreateMutable(NULL, 0);

	CFStringAppendCString(msg_text, from, 0);
	CFStringAppendCString(msg_text, sep, 0);
	CFStringAppendCString(msg_text, msg, 0);
	CFStringAppendCString(msg_text, "\r", 0);
	print_line(msg_text);
	CFRelease(msg_text);
}


void printMessage(char *from, char *message)
{
	outMessage(from,": ",message);
}

void printPrivMessage(char *from, char *message)
{
	outMessage(from,"> ",message);
}

void printOwnMessage(char *message)
{
	printMessage("",message);
}

void printSystemMessage(char *message)
{
	outMessage("","",message);
}



/*
 * Setzt durch ID referenzierten statischen Text
 *************************************************
 */
void setText(int the_id, char *the_text)
{
	ControlID k_text={SIGNATURE, the_id};
	ControlRef text;

	CFStringRef text_text;
	
	GetControlByID(static_win, &k_text, &text);
	
	text_text=CFStringCreateWithCString(NULL, the_text, 0);
	 
	SetControlData(text, 0, kControlEditTextCFStringTag, sizeof(CFStringRef), &text_text);
	CFRelease(text_text);
	
	DrawOneControl(text);
}




void setName(char *the_name)
{
	setText(ID_NAME, the_name);
}

void setNick(char *the_nick)
{
	setText(ID_NICK, the_nick);
}

void setChannel(char *the_channel)
{
	setText(ID_CHANNEL, the_channel);
}



// max. Laenge Eingabe / Buffer
#define MSG_MAX_LEN		250
#define MSG_BUF_LEN		(MSG_MAX_LEN+1)

/*
 * Wird vom Event-Handler nach Zeileneingabe oder Knopfdruck aufgerufen
 * Falls Text eingegeben wurde -> userInput() aufrufen
 ***********************************************************************
 */
void enter_text(WindowRef window)
{
	const ControlID k_textzeile={SIGNATURE, ID_LINE};
	
	ControlRef textzeile;

	CFStringRef zeile_text;
	GetControlByID(window, &k_textzeile, &textzeile);
	
	GetControlData(textzeile,0,kControlEditTextCFStringTag,sizeof(CFStringRef),&zeile_text,NULL);
	if(!CFStringGetLength(zeile_text))
		return;

	CFStringRef leer_text=CFSTR("");
	char textbuf[MSG_BUF_LEN];
	memset(textbuf,0,MSG_BUF_LEN);
	CFStringGetCString(zeile_text, textbuf, MSG_MAX_LEN, kCFStringEncodingMacRoman);

	SetControlData(textzeile, 0, kControlEditTextCFStringTag, sizeof(CFStringRef) ,&leer_text);
	DrawOneControl(textzeile);

	CFRelease(leer_text);
	
	userInput(textbuf);
}



/*
 * Kommando-Handler faengt Texteingabe und Knopfdruck ab
 *******************************************************
 */

static OSStatus cmd_handler(EventHandlerCallRef next_handler, EventRef event, void *user_data)
{
	HICommand cmd;
	WindowRef wnd=(WindowRef) user_data;
	OSStatus err=noErr;
	
	err=GetEventParameter(event,kEventParamDirectObject,typeHICommand,NULL,sizeof(HICommand),NULL,&cmd);
	require_noerr(err,CantGetParameter);
	
	switch(cmd.commandID)
	{
		case 'Send':
			enter_text(wnd);
			break;
			
		default:
			err=eventNotHandledErr;
	}
	CantGetParameter:
		return err;
}




/*
 * mainloop() wird als Timer-Event aufgerufen
 ********************************************
 */
void timer_loop(EventLoopTimerRef tref, void *user_data)
{
	mainloop();
}


/*
 * Timer fuer den Aufruf von mainloop() installieren
 * Erster Trigger nach 500ms, dann alle 20ms
 **************************************************
 */
OSStatus install_timer()
{
	EventLoopRef       mainLoop;
	EventLoopTimerUPP  timerUPP;
	EventLoopTimerRef  theTimer;

	mainLoop = GetMainEventLoop();
	timerUPP = NewEventLoopTimerUPP(timer_loop);

	InstallEventLoopTimer (mainLoop, 500*kEventDurationMillisecond, 20*kEventDurationMillisecond,
						   timerUPP, NULL, &theTimer);

}


/*
 * Event-Handler, faengt nur das Beenden auf
 **************************************************
 */
static OSStatus event_handler(EventHandlerCallRef next_handler, EventRef event, void *user_data)
{
	OSStatus err=noErr;
	
	switch(GetEventKind(event))
	{
		case kEventWindowClosed:
			QuitApplicationEventLoop();
			break;
		default:
			err=eventNotHandledErr;
			break;
	}
	return err;
}


/*
 * Fenster oeffnen, Timer+Event/Cmd-Handler installieren
 ******************************************************
 */
static OSStatus create_main_window()
{
	IBNibRef nib_ref;
	OSStatus err;
	
	const EventTypeSpec cmds[]={{kEventClassCommand, kEventCommandProcess}};
	const EventTypeSpec events[]={{kEventClassWindow, kEventWindowClosed}};
	
	err=CreateNibReference(CFSTR("main"), &nib_ref);
	require_noerr(err,CantGetNibRef);
	
	err=CreateWindowFromNib(nib_ref,CFSTR("IBRC_Window"),&static_win);
	require_noerr(err, CantCreateWindow);
	
	DisposeNibReference(nib_ref);
	
	err=InstallWindowEventHandler(static_win, NewEventHandlerUPP(event_handler),
			GetEventTypeCount(events), events, static_win, NULL);
	require_noerr(err, CantInstallHandler);
	
	err=InstallWindowEventHandler(static_win, NewEventHandlerUPP(cmd_handler),
			GetEventTypeCount(cmds), cmds, static_win, NULL);
	require_noerr(err, CantInstallHandler);
	

	ShowWindow(static_win);
	
	install_timer();
	
CantInstallHandler:
	CantCreateWindow:
	CantGetNibRef:
	
		return err;
}



/*
 * Programmaufruf mit <programm> <server> <port>
 ***********************************************
 */
int main(int argc, char* argv[])
{
    OSStatus		err;

	if(argc != 3)
	{
		fprintf(stderr, "Usage: %s <server> <port>\n", argv[0]);
		exit(1);
	}
	
	err=create_main_window();
	init(argv[1], atoi(argv[2]));
	RunApplicationEventLoop();
	return err;
}


void bye()
{
	QuitApplicationEventLoop();	
}

